
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
/**
 * Grafische Darstellung der Landkarte
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */
class Landkarte extends JComponent implements Konstanten
{
    /** Höhe für die Darstellung eines Raums. */
    private final int raumHöhe = 20;
    /** Breite für die Darstellung eines Raums. */
    private final int raumBreite = 20;
    /** Abstand zweier Räume in x-Richtung */
    private final int abstandX = 40;
    /** Abstand zweier Räume in y-Richtung */
    private final int abstandY = 40;
    /** Oberer Rand der Zeichnung */
    private final int karteOben = 5;
    /** Unterer Rand der Zeichnung */
    private final int karteUnten = karteOben + (raumHöhe *  höheRaumRaster + abstandY * (höheRaumRaster - 1));
    /** Linker Rand der Zeichnung */
    private final int karteLinks = 5;
    /** Rechter Rand der Zeichnung */
    private final int karteRechts = karteLinks + raumBreite *  breiteRaumRaster + abstandX * (breiteRaumRaster - 1);
    /** Daten der Räume */
    private ArrayList<LandkarteneintragRaum> räume;
    /** Daten der Wege */
    private ArrayList<LandkarteneintragWeg> wege;
    /** Raum, in dem die Maus gedrückt wurde oder -1 */
    private int raumGeklickt;
    /** Der Kontroller für die Verbindung zur Programmlogik */
    private Kontroller kontroller;

    /**
     * Legt die Datenstrukturen zum Speichern der Räume und Wege an
     */
    Landkarte(Kontroller k)
    {
        super();
        räume = new ArrayList<LandkarteneintragRaum>();
        wege = new ArrayList<LandkarteneintragWeg>();
        raumGeklickt = -1;
        kontroller = k;
        addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent e)
            {
                requestFocusInWindow();
                MausGedrückt(e);
            }
            public void mouseReleased(MouseEvent e)
            {
                MausLosgelassen(e);
            }
        });
    }
    
    
    /**
     * Sucht, ob es zum angegeben Punkt einen Raum gibt
     * @param p der Punkt, an dem die Maus gedrückt wurde
     * @return Raumnummer oder -1 für "kein Raum gefunden"
     */
    private int RaumSuchen(Point p)
    {
        for (LandkarteneintragRaum raum: räume)
        {
            Rectangle r = new Rectangle(karteLinks + (raumBreite + abstandX) *  raum.x, karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - raum.y), raumBreite, raumHöhe);
            if (r.contains(p))
            {
                return raum.nummer;
            }
        }
        return -1;
    }
    
    /**
     * Wird beim Drücken der Maus aufgerufen und bestimmt den (gegebenenfalls) gedrückten Raum
     * @param der Mausevent
     */
    void MausGedrückt(MouseEvent e)
    {
        raumGeklickt = RaumSuchen(e.getPoint());
    }
    
    /**
     * Wird beim Drücken der Maus aufgerufen und bestimmt den (gegebenenfalls) gedrückten Raum
     * @param der Mausevent
     */
    void MausLosgelassen(MouseEvent e)
    {
        if ((raumGeklickt == RaumSuchen(e.getPoint())) && (raumGeklickt != -1))
        {
            kontroller.KommandoEintragen(Kommandos.zimmerWechsel, raumGeklickt);
        }
        raumGeklickt = -1;
    }

    /**
     * Zeichnet einen Raum der Landkarte
     * @param raum die Raumdaten
     * @param g das Grafikobjekt
     */
    void RaumZeichnen(LandkarteneintragRaum raum, Graphics g)
    {
        int oben, links;
        Color oldForeground = g.getColor();
        oben = karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - raum.y);
        links = karteLinks + (raumBreite + abstandX) *  raum.x;
        switch (raum.mode)
        {
            case nichtBesucht:
                g.setColor(Color.lightGray);
                break;
            case besucht:
                g.setColor(Color.yellow);
                break;
            case ausgangGesucht:
                g.setColor(Color.green);
                break;
            case markiert:
                g.setColor(Color.red);
                break;
            default:
                g.setColor(Color.white);
        }
        g.fillRect(links, oben, raumBreite, raumHöhe);
        g.setColor(oldForeground);
        g.drawRect(links, oben, raumBreite, raumHöhe);
        g.drawString(Integer.toString(raum.nummer), links + 4, oben + 15);
    }
    
    /**
     * Zeichnet einen Weg der Landkarte
     * @param x1 x-Koordiante des Weganfangs
     * @param y1 y-Koordiante des Weganfangs
     * @param x2 x-Koordiante des Wegendes
     * @param y2 y-Koordiante des Wegendes
     * @param g das Grafikobjekt
     */
    void WegZeichnen(int x1, int y1, int x2, int y2, Graphics g)
    {
        int x, xl, xr, y, yu, yo;
        Color oldForeground = g.getColor ();
        g.setColor(Color.yellow);
        if (x1 == x2)
        {
            x = karteLinks + (raumBreite + abstandX) *  x1 - 1 + raumBreite / 2;
            yu = karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - y1) ;
            yo = karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - y2) + raumHöhe;
            g.drawLine(x, yo, x, yu);
            g.drawLine(x + 1, yo, x + 1, yu);
            g.setColor(oldForeground);
            g.drawLine(x - 1, yo, x - 1, yu);
            g.drawLine(x + 2, yo, x + 2, yu);
        }
        else if (y1 == y2)
        {
            xl = karteLinks + (raumBreite + abstandX) *  x1 + raumBreite;
            xr = karteLinks + (raumBreite + abstandX) *  x2;
            y = karteOben + (raumHöhe + abstandY) * (höheRaumRaster - 1 - y1) + raumHöhe / 2;
            g.drawLine(xl, y, xr, y);
            g.drawLine(xl, y + 1, xr, y + 1);
            g.setColor(oldForeground);
            g.drawLine(xl, y - 1, xr, y - 1);
            g.drawLine(xl, y + 2, xr, y + 2);
        }
        else if (y1 < y2)
        {
            xl = karteLinks + (raumBreite + abstandX) *  x1 + raumBreite;
            xr = karteLinks + (raumBreite + abstandX) *  x2;
            yu = karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - y1);
            yo = karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - y2) + raumHöhe;
            g.drawLine(xl, yu, xr, yo);
            g.drawLine(xl, yu + 1, xr + 1, yo);
            g.drawLine(xl - 1, yu, xr, yo - 1);
            g.setColor(oldForeground);
            g.drawLine(xl, yu + 2, xr + 2, yo);
            g.drawLine(xl - 2, yu, xr, yo - 2);
        }
        else
        {
            xl = karteLinks + (raumBreite + abstandX) *  x1 + raumBreite;
            xr = karteLinks + (raumBreite + abstandX) *  x2;
            yu = karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - y2);
            yo = karteOben + (raumHöhe + abstandY) *  (höheRaumRaster - 1 - y1) + raumHöhe;
            g.drawLine(xl, yo, xr, yu);
            g.drawLine(xl, yo - 1, xr + 1, yu);
            g.drawLine(xl - 1, yo, xr, yu + 1);
            g.setColor(oldForeground);
            g.drawLine(xl, yo - 2, xr + 2, yu);
            g.drawLine(xl - 2, yo, xr, yu + 2);
        }
    }
    
    /**
     * Zeichnet die Landkarte
     * @param g das Grafikobjekt
     */
    public void paint(Graphics g)
    {
        g.drawRect(karteLinks - 2, karteOben - 2, karteRechts - karteLinks + 3, karteUnten - karteOben + 3);
        
        for (LandkarteneintragRaum raum: räume)
        {
            RaumZeichnen(raum, g);
        }
        for (LandkarteneintragWeg weg: wege)
        {
            WegZeichnen(weg.x1, weg.y1, weg.x2, weg.y2, g);
        }
    }
    
    /**
     * Löscht die gespeicherten Landkartendaten
     */
    void LandkartendatenLöschen()
    {
        räume.clear();
        wege.clear();
    }
    
    /**
     * Trägt die Daten für einen weiteren Raum ein
     * @param r Raumdaten
     */
    void RaumEintragen(LandkarteneintragRaum r)
    {
        for (LandkarteneintragRaum raum: räume)
        {
            if (r.nummer == raum.nummer)
            {
                return;
            }
        }
        räume.add(r);
    }
    
    /**
     * Trägt die Daten für einen weiteren Weg ein
     * @param w Wegdaten
     */
    void WegEintragen(LandkarteneintragWeg w)
    {
        int hilf;
        if (w.x1 > w.x2)
        {
            hilf = w.x2;
            w.x2 = w.x1;
            w.x1 = hilf;
            hilf = w.y2;
            w.y2 = w.y1;
            w.y1 = hilf;
        }
        else if ((w.x1 == w.x2) && (w.y1 > w.y2))
        {
            hilf = w.y2;
            w.y2 = w.y1;
            w.y1 = hilf;
        }
        for (LandkarteneintragWeg weg: wege)
        {
            if ((w.x1 == weg.x1) && (w.x2 == weg.x2) &&
                (w.y1 == weg.y1) && (w.y2 == weg.y2))
            {
                return;
            }
        }
        wege.add(w);
    }
}
