
import java.util.*;
/**
 * Schafft den Rahmen für das Spiel.
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */
class Rahmen extends Ereignisbehandlung
{
    /** Die Seitenlänge der Basisquadrates des Spielfelds */
    final int basisGröße = 20;
    /** Anzahl der verstreuten Krümel. */
    int krümelAnzahl;
    /** Anzahl der gefressenen Krümel. */
    int gefressenAnzahl;
    /** Feld aller Hindernisse */
    private ArrayList<Hindernis> hindernisse;
    /** Das Monster */
    private Monster monster;
    /** Der Statustext */
    private Status status;
    /** Der Jäger */
    private Jaeger jäger;
    /** Der Chaot */
    private Chaot chaot;
    /** Der Fallensteller */
    private Fallensteller fallensteller;

    /**
     * Legt das Labyrinth, die Krümel und das Monster an
     */
    Rahmen ()
    {
        krümelAnzahl = 0;
        gefressenAnzahl = 0;
        hindernisse = new ArrayList<Hindernis>();
        for (int x = 0; x < 28; x += 1)
        {
            hindernisse.add(new MauerRechteck(x, 0, this));
            hindernisse.add(new MauerRechteck(x, 30, this));
        }
        for (int y = 1; y <= 8; y += 1)
        {
            hindernisse.add(new MauerRechteck(0, y, this));
            hindernisse.add(new MauerRechteck(27, y, this));
        }
        for (int x = 2; x <= 5; x += 1)
        {
            for (int y = 2; y <= 7; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 20, y, this));
            }
        }
        for (int x = 7; x <= 9; x += 1)
        {
            for (int y = 2; y <= 4; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 11, y, this));
            }
        }
        for (int x = 11; x <= 16; x += 1)
        {
            for (int y = 2; y <= 4; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
            }
        }
        for (int x = 0; x <= 5; x += 1)
        {
            for (int y = 9; y <= 13; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 22, y, this));
            }
        }
        for (int x = 7; x <= 20; x += 1)
        {
            hindernisse.add(new MauerRechteck(x, 6, this));
            hindernisse.add(new MauerRechteck(x, 7, this));
        }
        for (int x = 7; x <= 8; x += 1)
        {
            for (int y = 8; y <= 10; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 12, y, this));
            }
        }
        for (int x = 10; x <= 17; x += 1)
        {
            hindernisse.add(new MauerRechteck(x, 9, this));
            hindernisse.add(new MauerRechteck(x, 10, this));
        }
        for (int y = 12; y <= 16; y += 1)
        {
            hindernisse.add(new MauerRechteck(7, y, this));
            hindernisse.add(new MauerRechteck(8, y, this));
            hindernisse.add(new MauerRechteck(19, y, this));
            hindernisse.add(new MauerRechteck(20, y, this));
        }
        for (int x = 10; x <= 17; x += 1)
        {
            for (int y = 12; y <= 16; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
            }
        }
        for (int y = 15; y <= 29; y += 1)
        {
            hindernisse.add(new MauerRechteck(0, y, this));
            hindernisse.add(new MauerRechteck(27, y, this));
        }
        for (int x = 1; x <= 5; x += 1)
        {
            for (int y = 15; y <= 19; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 21, y, this));
            }
        }
        for (int x = 7; x <= 11; x += 1)
        {
            hindernisse.add(new MauerRechteck(x, 18, this));
            hindernisse.add(new MauerRechteck(x, 19, this));
            hindernisse.add(new MauerRechteck(x + 9, 18, this));
            hindernisse.add(new MauerRechteck(x + 9, 19, this));
        }
        for (int y = 18; y <= 22; y += 1)
        {
            hindernisse.add(new MauerRechteck(13, y, this));
            hindernisse.add(new MauerRechteck(14, y, this));
        }
        for (int x = 1; x <= 2; x += 1)
        {
            for (int y = 21; y <= 22; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 24, y, this));
            }
        }
        for (int x = 4; x <= 5; x += 1)
        {
            for (int y = 21; y <= 25; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 18, y, this));
            }
        }
        for (int x = 2; x <= 3; x += 1)
        {
            for (int y = 24; y <= 28; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 22, y, this));
            }
        }
        for (int x = 7; x <= 11; x += 1)
        {
            hindernisse.add(new MauerRechteck(x, 21, this));
            hindernisse.add(new MauerRechteck(x, 22, this));
            hindernisse.add(new MauerRechteck(x + 9, 21, this));
            hindernisse.add(new MauerRechteck(x + 9, 22, this));
        }
        for (int x = 10; x <= 17; x += 1)
        {
            hindernisse.add(new MauerRechteck(x, 24, this));
            hindernisse.add(new MauerRechteck(x, 25, this));
        }
        hindernisse.add(new MauerRechteck(10, 23, this));
        hindernisse.add(new MauerRechteck(11, 23, this));
        hindernisse.add(new MauerRechteck(16, 23, this));
        hindernisse.add(new MauerRechteck(17, 23, this));
        for (int x = 7; x <= 8; x += 1)
        {
            for (int y = 24; y <= 25; y += 1)
            {
                hindernisse.add(new MauerRechteck(x, y, this));
                hindernisse.add(new MauerRechteck(x + 12, y, this));
            }
        }
        for (int x = 5; x <= 22; x += 1)
        {
            hindernisse.add(new MauerRechteck(x, 27, this));
            hindernisse.add(new MauerRechteck(x, 28, this));
        }        
        
        monster = new Monster(4, 1, this, 3);
        
        for (int x = 1; x <= 26; x += 1)
        {
            hindernisse.add(new Kruemel(x, 1, this));
            krümelAnzahl = krümelAnzahl + 1;
        }
        for (int y = 2; y <= 8; y += 1)
        {
            hindernisse.add(new Kruemel(1, y, this));
            hindernisse.add(new Kruemel(26, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        for (int x = 7; x <= 20; x += 1)
        {
            hindernisse.add(new Kruemel(x, 5, this));
            krümelAnzahl = krümelAnzahl + 1;
        }
        for (int y = 2; y <= 4; y += 1)
        {
            hindernisse.add(new Kruemel(10, y, this));
            hindernisse.add(new Kruemel(17, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        for (int y = 2; y <= 25; y += 1)
        {
            hindernisse.add(new Kruemel(6, y, this));
            hindernisse.add(new Kruemel(21, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        for (int x = 2; x <= 5; x += 1)
        {
            hindernisse.add(new Kruemel(x, 8, this));
            hindernisse.add(new Kruemel(x + 20, 8, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        for (int x = 9; x <= 18; x += 1)
        {
            hindernisse.add(new Kruemel(x, 8, this));
            krümelAnzahl = krümelAnzahl + 1;
        }
        for (int x = 0; x < 5; x += 1)
        {
            hindernisse.add(new Kruemel(x + 1, 20, this));
            hindernisse.add(new Kruemel(x + 7, 20, this));
            hindernisse.add(new Kruemel(x + 16, 20, this));
            hindernisse.add(new Kruemel(x + 22, 20, this));
            krümelAnzahl = krümelAnzahl + 4;
        }
        for (int y = 20; y <= 23; y += 1)
        {
            hindernisse.add(new Kruemel(12, y, this));
            hindernisse.add(new Kruemel(15, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        for (int x = 4; x <= 23; x += 1)
        {
            hindernisse.add(new Kruemel(x, 26, this));
            krümelAnzahl = krümelAnzahl + 1;
        }
        for (int x = 1; x <= 26; x += 1)
        {
            hindernisse.add(new Kruemel(x, 29, this));
            krümelAnzahl = krümelAnzahl + 1;
        }
        for (int y = 23; y <= 28; y += 1)
        {
            hindernisse.add(new Kruemel(1, y, this));
            hindernisse.add(new Kruemel(26, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        for (int y = 21; y <= 23; y += 1)
        {
            hindernisse.add(new Kruemel(3, y, this));
            hindernisse.add(new Kruemel(24, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        hindernisse.add(new Kruemel(2, 23, this));
        hindernisse.add(new Kruemel(25, 23, this));
        krümelAnzahl = krümelAnzahl + 2;
        for (int y = 27; y <= 29; y += 1)
        {
            hindernisse.add(new Kruemel(4, y, this));
            hindernisse.add(new Kruemel(23, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        for (int y = 23; y <= 25; y += 1)
        {
            hindernisse.add(new Kruemel(9, y, this));
            hindernisse.add(new Kruemel(18, y, this));
            krümelAnzahl = krümelAnzahl + 2;
        }
        hindernisse.add(new Kruemel(7, 23, this));
        hindernisse.add(new Kruemel(8, 23, this));
        hindernisse.add(new Kruemel(19, 23, this));
        hindernisse.add(new Kruemel(20, 23, this));
        hindernisse.add(new Kruemel(13, 23, this));
        hindernisse.add(new Kruemel(14, 23, this));
        krümelAnzahl = krümelAnzahl + 6;
        
        status = new Status();
        status.AnzeigeSetzen(gefressenAnzahl, krümelAnzahl, monster.LebenGeben());
        
        jäger = new Jaeger (13, 11, this);
        hindernisse.add(jäger);
        chaot = new Chaot (14, 11, this);
        hindernisse.add(chaot);
        fallensteller = new Fallensteller (12, 11, this);
        hindernisse.add(fallensteller);

        Starten();
    }
    
    /**
     * Meldet eine Kopie des Felds der Hindernisse zurück;
     */
    ArrayList<Hindernis> HindernisseGeben()
    {
        return new ArrayList<Hindernis>(hindernisse);
    }
    
    void KrümelEntfernen (Hindernis krümel)
    {
        gefressenAnzahl = gefressenAnzahl + 1;
        hindernisse.remove(krümel);
        status.AnzeigeSetzen(gefressenAnzahl, krümelAnzahl, monster.LebenGeben());
        if (gefressenAnzahl == krümelAnzahl)
        {
            new Gewonnen();
            Anhalten();
        }
    }
    
    /**
     * Wird vom Monster aufgerufen, wenn die Anzahl der Leben verändert wurde.
     */
    void StatusÄndern()
    {
        status.AnzeigeSetzen(gefressenAnzahl, krümelAnzahl, monster.LebenGeben());
        if (monster.LebenGeben() <= 0)
        {
            new Verloren();
            Anhalten();
        }
    }
    
    /**
     * Meldet das Monster
     * @return Monster
     */
    Monster MonsterGeben()
    {
        return monster;
    }
    
    /**
     * Meldet, ob auf dem angegebenen Feld ein Mauerstein steht
     * @param x x-Position des Feldes
     * @param y y-Position des Feldes
     * @return wahr, wenn auf dem Feld ein Mauerstein steht
     */
    boolean IstMauer (int x, int y)
    {
        if ((x <= 0) || (x >= 28 * basisGröße) || (y <= 0) || (y >= 31 * basisGröße))
        {
            return true;
        }
        for (Hindernis h: hindernisse)
        {
            if ((h instanceof MauerRechteck) && (x == h.XPositionGeben()) &&  (y == h.YPositionGeben()))
            {
                return true;
            }
        }
        return false;
    }
    
    /**
     * nichts
     */
    @Override void SonderTasteGedrückt(int taste)
    {
    }
    
    /**
     * nichts
     */
    @Override void TasteGedrückt(char taste)
    {
    }
    
    /**
     * nichts
     */
    @Override void TaktImpulsAusführen()
    {
    }
}
