
/**
 * Ein Fahrzeugsymbol
 *
 * @author Albert Wiedemann
 * @version 1.0
 */
class LKWMitAnhaengerSymbol extends FahrzeugSymbol
{
    /** Interna */
    private Rechteck aufbau;
    private Rechteck aufbauHänger;
    private Rechteck kupplung;
    private Rechteck scheibe;

    /**
     * Standardkonstruktor für Objekte der Klasse FahrzeugSymbol.
     */
    LKWMitAnhaengerSymbol()
    {
        super();
        länge = 66;
        breite = 9;
        aufbau = new Rechteck();
        aufbau.PositionSetzen(0, 0);
        aufbau.GrößeSetzen(breite, länge/2);
        aufbau.FarbeSetzen(aktFarbe);
        scheibe = new Rechteck();
        scheibe.PositionSetzen(1, länge/12);
        scheibe.GrößeSetzen(breite - 2, länge/12);
        scheibe.FarbeSetzen("weiß");
        aufbauHänger = new Rechteck();
        aufbauHänger.PositionSetzen(0, länge*7/12);
        aufbauHänger.GrößeSetzen(breite, länge - länge*7/12);
        aufbauHänger.FarbeSetzen(aktFarbe);
        kupplung = new Rechteck();
        kupplung.PositionSetzen(breite/2 - 1, länge/2);
        kupplung.GrößeSetzen(3, länge/12);
        kupplung.FarbeSetzen("schwarz");
    }

    /**
     * Setzt die Farbe des Fahrzeugsymbols.
     * @param neueFarbe die neue Farbe des Fahrzeugsymbols
     */
    @Override void FarbeSetzen(String neueFarbe)
    {
        super.FarbeSetzen(neueFarbe);
        aufbau.FarbeSetzen(aktFarbe);
        aufbauHänger.FarbeSetzen(aktFarbe);
    }

    /**
     * Setzt die Position des Fahrzeugsymbols.
     * Die Koordinaten bezeichnen die linke, obere Ecke des Fahrzeugsymbols.
     * @param x x-Position
     * @param y y-Position
     */
    @Override void PositionSetzen(int x, int y)
    {
        super.PositionSetzen(x, y);
        aufbau.PositionSetzen(x, y);
        scheibe.PositionSetzen(x + 1, y + länge/12);
        aufbauHänger.PositionSetzen(x, y + länge*7/12);
        kupplung.PositionSetzen(x + breite/2 - 1, y + länge/2);
    }

    /**
     * Entfernt das Fahrzeugsymbol aus der Anzeige
     */
    @Override void Entfernen()
    {
        aufbau.Entfernen();
        scheibe.Entfernen();
        aufbauHänger.Entfernen();
        kupplung.Entfernen();
    }
}
