
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
/**
 * Bedienoberfläche für das Smart Home
 *
 * @author Albert Wiedemann
 * @version 1.0
 */
class Oberflaeche
{
    /** Das Bedienfenster */
    JFrame anzeige;

    /**
     * Legt das Oberflächenfenster an
     * @param anzahlKomponenten die geplante Anzahl der anzuzeigenden Komponenten
     */
    Oberflaeche(int anzahlKomponenten)
    {
        anzeige = new JFrame("Übersicht");
        anzeige.setLocation(20, 20);
        anzeige.setSize(800, 660);
        anzeige.setLayout(new GridLayout​(5, anzahlKomponenten / 5 + 1, 5, 5));
    }

    /**
     * Fügt eine neue Komponente in das Fenster ein
     * @param gerätename Name der Komponente
     * @param startZustand Anfangszustand für die Anzeige
     * @param z hier Oberfläche als Änderungsbeobachter registrieren
     */
    void SmartKomponenteHinzufügen(String gerätename, String startZustand, Smartkomponente z)
    {
        KomponentenPanel p;
        if (z instanceof WertEmpfaenger)
        {
            p = new MessgerätePanel(gerätename, startZustand, (WertEmpfaenger) z);
        }
        else if (z instanceof AktionsEmpfaenger)
        {
            p = new HausgerätePanel(gerätename, startZustand, (AktionsEmpfaenger) z);
        }
        else
        {
            p = null;
        }
        z.BeobachterRegistrieren(p);
        anzeige.add(p);
    }
    
    /**
     * Obefläche am Bildschirm anzeigen
     */
    void Anzeigen()
    {
        anzeige.setVisible(true);
    }
}

/**
 * Oberklasse für die Anzeigepanele der Komponenten
 */
class KomponentenPanel extends JPanel implements ZustandsBeobachter
{
    /** Darstellungskomponenten */
    private JLabel name;
    private JLabel zustand;
    
    /**
     * Legt Name und Startzustand fest
     * @param gerätename Name der Komponente
     * @param startZustand Anfangszustand für die Anzeige
     */
    KomponentenPanel (String gerätename, String startZustand)
    {
        super(null);
        setLocation(0, 0);
        setVisible(true);
        setSize(200, 140);
        name = new JLabel(gerätename);
        name.setLocation(0, 0);
        name.setVisible(true);
        name.setSize(200, 20);
        add(name);
        zustand = new JLabel(startZustand);
        zustand.setLocation(0, 20);
        zustand.setVisible(true);
        zustand.setSize(200, 20);
        add(zustand);
    }
    
    /**
     * Wird bei Zustandsänderungen aufgerufen
     * @param zustandNeu der neue Zustand
     */
    public void ÄnderungMelden(String zustandNeu)
    {
        zustand.setText(zustandNeu);
    }
}

/**
 * Anzeigepanel für Sensoren
 */
class MessgerätePanel extends KomponentenPanel
{
    /** Darstellungskomponenten */
    private JTextField eingabe;
    private WertEmpfaenger empfänger;
    
    /**
     * Baut die Ansicht auf
     * @param gerätename Name der Komponente
     * @param startZustand Anfangszustand für die Anzeige
     * @param e Empfänger für die Sensorwerte
     */
    MessgerätePanel (String gerätename, String startZustand, WertEmpfaenger e)
    {
        super(gerätename, startZustand);
        empfänger = e;
        JLabel l = new JLabel("Neuer Wert");
        l.setLocation(0, 40);
        l.setVisible(true);
        l.setSize(200, 20);
        add(l);
        eingabe = new JTextField();
        eingabe.setLocation(0, 60);
        eingabe.setVisible(true);
        eingabe.setSize(200, 30);
        add(eingabe);
        JButton b = new JButton("Setzen");
        b.setLocation(10, 90);
        b.setVisible(true);
        b.setSize(180, 30);
        add(b);
        b.addActionListener(new ActionListener()
        {
            public void actionPerformed (ActionEvent e)
            {
                String text = eingabe.getText();
                if (empfänger.EingabePrüfen(text))
                {
                    empfänger.WertSetzen(text);
                }
            }
        });
    }
}

/**
 * Anzeigepanel für zu steuernde Hausgeräte
 */
class HausgerätePanel extends KomponentenPanel
{
    /** Der Empfänger der ausgelösten Aktionen */
    private AktionsEmpfaenger empfänger;
    
    /**
     * Baut die Ansicht auf
     * @param gerätename Name der Komponente
     * @param startZustand Anfangszustand für die Anzeige
     * @param e Empfänger für direkte Steuerung
     */
    HausgerätePanel (String gerätename, String startZustand, AktionsEmpfaenger e)
    {
        super(gerätename, startZustand);
        empfänger = e;
        JButton b = new JButton("Automatik");
        b.setLocation(10, 40);
        b.setVisible(true);
        b.setSize(180, 30);
        add(b);
        b.addActionListener(new ActionListener()
        {
            public void actionPerformed (ActionEvent e)
            {
                empfänger.AutomatikAktivieren();
            }
        });
        b = new JButton("An");
        b.setLocation(10, 70);
        b.setVisible(true);
        b.setSize(80, 30);
        add(b);
        b.addActionListener(new ActionListener()
        {
            public void actionPerformed (ActionEvent e)
            {
                empfänger.Einschalten();
            }
        });
        b = new JButton("Aus");
        b.setLocation(110, 70);
        b.setVisible(true);
        b.setSize(80, 30);
        add(b);
        b.addActionListener(new ActionListener()
        {
            public void actionPerformed (ActionEvent e)
            {
                empfänger.Ausschalten();
            }
        });
    }
}

