// WARNING: This file is auto-generated and any changes to it will be overwritten
import lang.stride.*;
import java.util.ArrayList;

/**
 * Verwaltet die Fahrzeuge auf einer Fahrbahn.
 * @author Albert Wiedemann @version V1.0
 */
public class Fahrbahn
{
    private int x;
    private int y;
    private float start;
    private float ende;
    private float rand;
    private char ausrichtung;
    private ArrayList<Fahrzeug> fahrzeuge;
    private FahrbahnSymbol darstellung;
    private Ampel ampel;

    /**
     * Erzeugt eine leere Fahrbahn ab dem linken Rand unterhalb der Fenstermitte. @param a für diese Richtung zuständige Ampel
     */
    protected Fahrbahn(Ampel a)
    {
        x = 0;
        y = 0;
        ausrichtung = 'O';
        ampel = a;
        fahrzeuge = new ArrayList < Fahrzeug > ( );
        darstellung = new FahrbahnSymbol ( );
        darstellung . PositionSetzen ( x , y );
        darstellung . AusrichtungSetzen ( ausrichtung );
        start = darstellung . EintrittsPositionGeben ( );
        ende = darstellung . AustrittsPositionGeben ( );
        rand = darstellung . RandAbstandGeben ( );
    }

    /**
     * Reiht ein neues Fahrzeug an der Startposition in die Fahrbahn ein. Das Fahrzeug wird nicht eingereiht, wenn kein Platz vorhanden ist. @param f das neue Fahrzeug
     */
    protected void FahrzeugEinreihen(Fahrzeug f)
    {
        fahrzeuge . add ( f );
        f . AusrichtungSetzen ( ausrichtung );
        if (( ausrichtung == 'O' ) || ( ausrichtung == 'W' )) {
            f . PositionSetzen ( start , rand );
        }
        else {
            f . PositionSetzen ( rand , start );
        }
    }

    /**
     * Ermittelt, ob ein Fahrzeug entfernt werden muss @param f das in Frage kommende Fahrzeug
     */
    private boolean IstDraussen(Fahrzeug f)
    {
        switch (ausrichtung) {
            case 'O' : {
                return f . XKoordinateGeben ( ) > ende;
            }
            case 'W' : {
                return f . XKoordinateGeben ( ) < ende;
            }
            case 'N' : {
                return f . YKoordinateGeben ( ) < ende;
            }
            case 'S' : {
                return f . YKoordinateGeben ( ) > ende;
            }
            default : {
                return false;
            }
        }
    }

    /**
     * Ermittelt, ob das Fahrzeug sich nach dem Vorgänger richten oder die Ampel beachten muss. @return true, wenn der Vorgänger wichtig ist, false, wenn die Ampel zählt
     */
    private boolean VorgängerZählt(float ampelabstand, float vorgängerabstand, Fahrzeug f)
    {
        return ( ampelabstand < 0.0f ) || ( vorgängerabstand < ampelabstand ) || ampel . IstGrün ( ) || ampel . IstGelb ( ) && ( ampelabstand < f . GeschwindigkeitGeben ( ) * 1.8f ) && ( f . GeschwindigkeitGeben ( ) > 3.0f );
    }

    /**
     * Bewegt alle Fahrzeuge der Fahrbahn für eine Sekunde.
     */
    protected void FahrzeugeBewegen()
    {
        Fahrzeug vorgänger;
        float vvorgänger;
        float wegmax;
        float ampelAbstand;
        vorgänger = null;
        for (final Fahrzeug f : fahrzeuge) {
            if (vorgänger == null) {
                wegmax = 100000;
                vvorgänger = 100;
            }
            else {
                vvorgänger = vorgänger . GeschwindigkeitGeben ( );
                switch (ausrichtung) {
                    case 'O' : {
                        wegmax = vorgänger . XKoordinateGeben ( ) - f . XKoordinateGeben ( ) - f . LängeGeben ( );
                        break;
                    }
                    case 'W' : {
                        wegmax = f . XKoordinateGeben ( ) - vorgänger . XKoordinateGeben ( ) - vorgänger . LängeGeben ( );
                        break;
                    }
                    case 'N' : {
                        wegmax = f . YKoordinateGeben ( ) - vorgänger . YKoordinateGeben ( ) - vorgänger . LängeGeben ( );
                        break;
                    }
                    case 'S' : {
                        wegmax = vorgänger . YKoordinateGeben ( ) - f . YKoordinateGeben ( ) - f . LängeGeben ( );
                        break;
                    }
                    default : {
                        wegmax = 100000;
                    }
                }
                if (vvorgänger > 0) {
                    wegmax = wegmax - Math . round ( ( f . GeschwindigkeitGeben ( ) + vvorgänger ) / 2.0 * 1.8 );
                }
            }
            switch (ausrichtung) {
                case 'O' : {
                    ampelAbstand = darstellung . HaltelinienPositionGeben ( ) - f . XKoordinateGeben ( ) - f . LängeGeben ( );
                    break;
                }
                case 'W' : {
                    ampelAbstand = f . XKoordinateGeben ( ) - darstellung . HaltelinienPositionGeben ( );
                    break;
                }
                case 'N' : {
                    ampelAbstand = f . YKoordinateGeben ( ) - darstellung . HaltelinienPositionGeben ( );
                    break;
                }
                case 'S' : {
                    ampelAbstand = darstellung . HaltelinienPositionGeben ( ) - f . YKoordinateGeben ( ) - f . LängeGeben ( );
                    break;
                }
                default : {
                    ampelAbstand = 100000;
                }
            }
            if (VorgängerZählt ( ampelAbstand , wegmax , f )) {
                f . Fahren ( wegmax , vvorgänger );
            }
            else {
                f . Fahren ( ampelAbstand , 0.0f );
            }
            vorgänger = f;
        }
        if (( fahrzeuge . size ( ) > 0 ) && IstDraussen ( fahrzeuge . get ( 0 ) )) {
            fahrzeuge . get ( 0 ) . Entfernen ( );
            fahrzeuge . remove ( 0 );
        }
    }

    /**
     * Setzt die Position der Fahrbahn.
     * @param x x-Position @param y y-Position
     */
    protected void PositionSetzen(int x, int y)
    {
        this . x = x;
        this . y = y;
        darstellung . PositionSetzen ( x , y );
        rand = darstellung . RandAbstandGeben ( );
    }

    /**
     * Setzt die Ausrichtung des Fahrzeugs.
     * @param ausrichtung neue Ausrichtung
     */
    protected void AusrichtungSetzen(char ausrichtung)
    {
        this . ausrichtung = ausrichtung;
        darstellung . AusrichtungSetzen ( ausrichtung );
        start = darstellung . EintrittsPositionGeben ( );
        ende = darstellung . AustrittsPositionGeben ( );
        rand = darstellung . RandAbstandGeben ( );
    }

    /**
     * Ausgabe der Fahrzeuganzahl @return Anzahl der Fahrzeuge auf der Fahrbahn
     */
    protected int FahrzeuganzahlGeben()
    {
        return fahrzeuge . size ( );
    }
}
