
/**
 * Ein Fahrzeugsymbol auf einem Fenster.
 * Das Fenster wird bei Bedarf generiert.
 *
 * @author Albert Wiedemann
 * @version 1.0
 */

import java.awt.*;

abstract class FahrzeugSymbol
{
    /** Das Anzeigefenster.*/
    protected Frame fenster;
    
    /** Anzeigegröße für ein Fahrzeug */
    protected int länge;
    protected static final int breite = Oberflaeche.RasterGrößeGeben () / 2;

    /** Interna */
    protected String aktFarbe;
    protected float x;
    protected float y;
    protected Canvas anzeige;
    protected char ausrichtung;

    /**
     * Standardkonstruktor für Objekte der Klasse FahrzeugSymbol.
     * Keine Anzeige, Fahrtrichtung Osten.
     * Das Fenster wird bei Bedarf angelegt.
     */
    FahrzeugSymbol ()
    {
        länge = Oberflaeche.RasterGrößeGeben ();
        fenster = Oberflaeche.FensterGeben ();
        anzeige = null;
        aktFarbe = "blau";
        ausrichtung = 'O';
    }

    protected java.awt.Color AWTFarbe (String farbe)
    {
        if (farbe == "weiss")
        {
            return java.awt.Color.white;
        }
        else if (farbe == "schwarz")
        {
            return java.awt.Color.black;
        }
        else if (farbe == "rot")
        {
            return java.awt.Color.red;
        }
        else if ((farbe == "grün") || (farbe == "gr\u00FCn"))
        {
            return java.awt.Color.green;
        }
        else if (farbe == "blau")
        {
            return java.awt.Color.blue;
        }
        else if (farbe == "gelb")
        {
            return java.awt.Color.yellow;
        }
        else if (farbe == "magenta")
        {
            return java.awt.Color.magenta;
        }
        else if (farbe == "cyan")
        {
            return java.awt.Color.cyan;
        }
        else if (farbe == "grau")
        {
            return java.awt.Color.gray;
        }
        else
        {
           return java.awt.Color.white;
        }
    }

    /**
     * Zeichnet das Fahrzeugsymbol.
     */
    private void Zeichnen ()
    {
        anzeige.invalidate ();
        anzeige.repaint ();
    }

    /**
     * Setzt die Farbe des Fahrzeugsymbols.
     * @param neueFarbe die neue Farbe des Fahrzeugsymbols
     */
    void FarbeSetzen (String neueFarbe)
    {
        aktFarbe = neueFarbe.toLowerCase ();
        Zeichnen ();
    }

    /**
     * Setzt die Position des Fahrzeugsymbols.
     * Das Koordinatensystem rechnet in Metern; der Ursprung ist in der Mitte des Fensters.
     * Das Fahrzeugsymbol repräsentiert eine Länge von 5,0m.
     * Die Koordinaten bezeichnen die linke, obere Ecke des Fahrzeugsymbols.
     * @param x x-Position
     * @param y y-Position
     */
    void PositionSetzen (float x, float y)
    {
        this.x = x;
        this.y = y;
        anzeige.setLocation (Oberflaeche.FensterBreiteGeben () / 2 + (int) (x * Oberflaeche.MeterAlsPixelGeben ()), Oberflaeche.FensterHöheGeben () / 2 + (int) (y * Oberflaeche.MeterAlsPixelGeben ()));
    }

    /**
     * Setzt die Ausrichtung des Fahrzeugs.
     * 
     * @param ausrichtung Ausrichtung des Fahrzeugs.
     */
    void AusrichtungSetzen (char ausrichtung)
    {
        this.ausrichtung = ausrichtung;
        switch (ausrichtung)
        {
            case 'N':
                anzeige.setSize (breite, länge);
                break;
            case 'S':
                anzeige.setSize (breite, länge);
                break;
            case 'O':
                anzeige.setSize (länge, breite);
                break;
            case 'W':
                anzeige.setSize (länge, breite);
                break;
        }
        Zeichnen();
    }

    /**
     * Entfernt das Fahrzeugsymbol aus der Anzeige
     */
    public void Entfernen ()
    {
        fenster.remove (anzeige);
    }
    
    /**
     * Erzeugt ein neues Fahrzeugsymbol.
     * Mögliche Werte für den Parameter sind:
     * pkw: Standardpkw
     * lkw: Lkw
     * sportwagen: Sportwagen
     * lkwmitanhänger: Lkw mit Anhänger
     * @param art Art des zu erzeugenden Fahrzeugs
     */
    static FahrzeugSymbol FahrzeugSymbolErzeugen (String art)
    {
        if (art == "pkw")
        {
            return new PKWSymbol ();
        }
        else if (art == "lkw")
        {
            return new LKWSymbol ();
        }
        else if (art == "sportwagen")
        {
            return new SportwagenSymbol ();
        }
        else if (art == "lkwmitanhänger")
        {
            return new LKWMitAnhaengerSymbol ();
        }
        else
        {
            return null;
        }
    }
}
