
/**
 * Verwaltet das Ausgabefenster.
 * 
 * @author Albert Wiedemann
 * @version 1.0
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class Oberflaeche
{
    /** Höhe des Anzeigefensters */
    private static final int höheFenster = 800;
    /** Breite des Anzeigefensters */
    private static final int breiteFenster = 1000;
    /** Anzeigegröße für eine Lampe */
    private static final int rasterGröße = 20;
    /** Umrechnung Meter nach Pixel */
    private static final float meter2pixel = 5.0f;
    private static Oberflaeche o = null;
    private JFrame fenster = null;
    private JTextField eingabe;
    private JTextField eingabe2;
    private KontrolleurInterface k;

    /**
     * Baut die Bedienoberfläche auf
     */
    private Oberflaeche()
    {
        JLabel l;
        JButton button;
        fenster = new JFrame("Zeichenfläche");
        fenster.setResizable(false);
        fenster.setSize(breiteFenster, höheFenster + 100);
        fenster.setVisible(true);
        fenster.setLayout(null);
        fenster.setBackground(Color.white);
        fenster.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                System.exit(1);
            }
        });
        JComponent hintergrund = new JComponent()
        {
            /**
             * Zeichnet das Hintergrundmuster.
             */
            public void paint(Graphics g)
            {
                g.setColor(Color.lightGray);
                for (int i = 1; i <= höheFenster / (rasterGröße * 2); i++)
                {
                    g.drawLine(0, höheFenster / 2 + i * rasterGröße, breiteFenster - 1, höheFenster / 2 + i * rasterGröße);
                    g.drawLine(0, höheFenster / 2 - i * rasterGröße, breiteFenster - 1, höheFenster / 2 - i * rasterGröße);
                }
                for (int i = 1; i <= breiteFenster / (rasterGröße * 2); i++)
                {
                    g.drawLine(breiteFenster / 2 + i * rasterGröße, 0, breiteFenster / 2 + i * rasterGröße, höheFenster - 1);
                    g.drawLine(breiteFenster / 2 - i * rasterGröße, 0, breiteFenster / 2 - i * rasterGröße, höheFenster - 1);
                }
                g.setColor(Color.black);
                g.drawLine(0, höheFenster / 2, breiteFenster - 1, höheFenster / 2);
                g.drawLine(breiteFenster / 2, 0, breiteFenster / 2, höheFenster - 1);
            }
        };
        hintergrund.setVisible(true);
        hintergrund.setSize(breiteFenster, höheFenster);
        hintergrund.setLocation(0, 0);
        fenster.add(hintergrund);
        l = new JLabel();
        l.setText("Mittlerer Fahrzeugabstand");
        l.setLocation(20, höheFenster + 15);
        l.setVisible(true);
        l.setSize(200, 20);
        fenster.add(l);
        eingabe = new JTextField();
        eingabe.setLocation(220, höheFenster + 15);
        eingabe.setText("6");
        eingabe.setVisible(true);
        eingabe.setSize(60, 20);
        eingabe.setEnabled(true);
        fenster.add(eingabe);
        button = new JButton();
        button.setLocation(300, höheFenster + 10);
        button.setLabel("Setzen");
        button.setVisible(true);
        button.setSize(80, 30);
        button.setEnabled(true);
        button.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    k.FahrzeugabstandSetzen(Integer.parseInt(eingabe.getText()));
                }
                catch (Exception ex)
                {
                    (Toolkit.getDefaultToolkit()).beep();
                }
            }
        });
        fenster.add(button);

        l = new JLabel();
        l.setText("Taktlänge in Millisekunden");
        l.setLocation(400, höheFenster + 15);
        l.setVisible(true);
        l.setSize(200, 20);
        fenster.add(l);
        eingabe2 = new JTextField();
        eingabe2.setLocation(600, höheFenster + 15);
        eingabe2.setText("1000");
        eingabe2.setVisible(true);
        eingabe2.setSize(60, 20);
        eingabe2.setEnabled(true);
        fenster.add(eingabe2);
        button = new JButton();
        button.setLocation(680, höheFenster + 10);
        button.setLabel("Setzen");
        button.setVisible(true);
        button.setSize(80, 30);
        button.setEnabled(true);
        button.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    k.TaktdauerSetzen(Integer.parseInt(eingabe2.getText()));
                }
                catch (Exception ex)
                {
                    (Toolkit.getDefaultToolkit()).beep();
                }
            }
        });
        fenster.add(button);

        button = new JButton();
        button.setLocation(800, höheFenster + 10);
        button.setLabel("Starten");
        button.setVisible(true);
        button.setSize(80, 30);
        button.setEnabled(true);
        button.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                k.Starten();
            }
        });
        fenster.add(button);
        button = new JButton();
        button.setLocation(900, höheFenster + 10);
        button.setLabel("Stoppen");
        button.setVisible(true);
        button.setSize(80, 30);
        button.setEnabled(true);
        button.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                k.Anhalten();
            }
        });
        fenster.add(button);
    }

    /**
     * Gibt das Ausgabefenster zurück und erzeugt es gegebenenfalls.
     * @return Ausgabefenster
     */
    static JFrame FensterGeben()
    {
        if (o == null)
        {
            o = new Oberflaeche();
        }
        return o.fenster;
    }
    
    /**
     * Setzt Kreuzung und Taktgeber
     * @param k Kontrolleur
     */
    static void KontrolleurSetzen(KontrolleurInterface k)
    {
        if (o == null)
        {
            o = new Oberflaeche();
        }
        o.k = k;
    }

    /**
     * Gibt die Rastergröße zurück.
     * @return Ausgabefenster
     */
    static int RasterGrößeGeben()
    {
        return rasterGröße;
    }

    /**
     * Gibt die Breite des Fensters zurück.
     * @return Fensterbreite
     */
    static int FensterBreiteGeben()
    {
        return breiteFenster;
    }

    /**
     * Gibt die Höhe des Fensters zurück.
     * @return Fensterhöhe
     */
    static int FensterHöheGeben()
    {
        return höheFenster;
    }

    /**
     * Gibt die Höhe des Fensters zurück.
     * @return Fensterhöhe
     */
    static float MeterAlsPixelGeben()
    {
        return meter2pixel;
    }
}
